@@ -291,4 +291,12 @@ class ConfigTest extends \PHPUnit_Framework_TestCase
         );
         $this->assertTrue($config->get('disable-tls'));
     }
+
+    public function testProcessTimeout()
+    {
+        putenv('COMPOSER_PROCESS_TIMEOUT=0');
+        $config = new Config(true);
+        $this->assertEquals(0, $config->get('process-timeout'));
+        putenv('COMPOSER_PROCESS_TIMEOUT');
+    }
 }
